@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2016-2017, 2019-2020 eGauge Systems LLC
+# Copyright (c) 2016-2017, 2019-2021 eGauge Systems LLC
 # 	1644 Conestoga St, Suite 2
 # 	Boulder, CO 80301
 # 	voice: 720-545-9767
@@ -31,12 +31,14 @@
 import argparse
 import logging
 import os
+import re
 import subprocess
 import sys
 import tempfile
 
 import importlib_resources
 import pexpect
+import pkg_resources
 
 from pexpect import fdpexpect
 
@@ -59,11 +61,21 @@ from ctid_programmer.gui.main_window import Ui_MainWindow
 from ctid_programmer.gui.template_dialog import Ui_Template_Dialog
 from ctid_programmer.gui.preferences_dialog import Ui_Preferences_Dialog
 from ctid_programmer.gui.param_ct import Ui_Param_CT
-from ctid_programmer.gui.param_volt import Ui_Param_Volt
+from ctid_programmer.gui.param_linear import Ui_Param_Linear
 from ctid_programmer.gui.param_temp import Ui_Param_Temp
 from ctid_programmer.gui.param_ntc import Ui_Param_NTC
 from ctid_programmer.gui.param_pulse import Ui_Param_Pulse
 
+
+PACKAGE_NAME = 'CTid-programmer'
+PACKAGE_VERSION = pkg_resources.get_distribution('CTid-programmer').version
+VERSION_INFO = f'''{PACKAGE_NAME} {PACKAGE_VERSION}
+Copyright (c) 2018-2022 eGauge Systems LLC
+License MIT: <https://opensource.org/licenses/MIT>.
+This is free software: you are free to change and redistribute it.
+
+Written by Alexandra Kaufhold and David Mosberger.'''
+
 PATH_AVRDUDE = 'avrdude'
 PATH_CTID_ENCODER = 'ctid-encoder'
 
@@ -73,7 +85,7 @@ SENSOR_LONG_NAME = {
     'AC': 'AC Current Sensor',
     'DC': 'DC Current Sensor',
     'RC': 'Rogowski Coil Sensor',
-    'voltage': 'Voltage Sensor',
+    'linear': 'Linear Sensor',
     'temp': 'Linear Temperature Sensor',
     'NTC': 'NTC Thermistor Sensor',
     'pulse': 'Pulse Sensor'
@@ -224,7 +236,7 @@ class Template_List_Dialog(QDialog, Ui_Template_Dialog):
                                               'that template?' % new_name,
                                               QMessageBox.Yes,
                                               QMessageBox.Cancel)
-                if choice == 1:
+                if choice == QMessageBox.Cancel:
                     return	# let user correct name
             else:
                 self.add_template(new_name)
@@ -258,7 +270,7 @@ class Preferences_Editor(QDialog, Ui_Preferences_Dialog):
                     break
         self.increment_spinbox.setValue(self.parent.preferences.sn_increment)
         self.station_id_spinbox.setValue(self.parent.preferences.station_id)
-        super().exec_()
+        return super().exec_()
 
     def accept(self):
         increment = self.increment_spinbox.value()
@@ -296,9 +308,9 @@ class UI(QMainWindow, Ui_MainWindow):
         ct_params = sensor_params.CT(Ui_Param_CT(), self.param_group,
                                      'CT Parameters')
         self.params['AC'] = self.params['DC'] = self.params['RC'] = ct_params
-        self.params['voltage'] = sensor_params.Volt(Ui_Param_Volt(),
-                                                    self.param_group,
-                                                    'Voltage Parameters')
+        self.params['linear'] = sensor_params.Linear(Ui_Param_Linear(),
+                                                     self.param_group,
+                                                     'Linear Parameters')
         self.params['temp'] = sensor_params.Temp(Ui_Param_Temp(),
                                                  self.param_group,
                                                  'Temperature Parameters')
@@ -340,6 +352,7 @@ class UI(QMainWindow, Ui_MainWindow):
             name = SENSOR_LONG_NAME[st] if st in SENSOR_LONG_NAME else st
             self.sensor_type_combo.addItem(name)
 
+        self.action_About.triggered.connect(self.about)
         self.sensor_type_combo.currentIndexChanged.connect(
             self.sensor_type_changed)
         self.mfg_combo.currentIndexChanged.connect(self.product_changed)
@@ -359,6 +372,9 @@ class UI(QMainWindow, Ui_MainWindow):
 
         self.prefs_changed()
 
+    def about(self):
+        QMessageBox.about(self, 'About', VERSION_INFO)
+
     def log(self, msg):
         if self.welcome_msg:
             self.plainTextEdit.clear()
@@ -380,6 +396,11 @@ class UI(QMainWindow, Ui_MainWindow):
 
             self.product_changed()
 
+            # 'voltage' sensor type is now called 'linear':
+            if template['sensor_type'] == 'voltage':
+                template['sensor_type'] = 'linear'
+                template['unit'] = 0
+
             self.sensor_type_combo.setCurrentIndex(0)
             for idx, code in enumerate(ctid.SENSOR_TYPE_NAME):
                 if code == template['sensor_type']:
@@ -658,21 +679,28 @@ class UI(QMainWindow, Ui_MainWindow):
         self.log('Detected %s chip.' % chip[1])
         return chip[0]
 
-    def write_flash(self, chip_type, hexfile, cleaned_data):
+    def write_flash(self, chip_type, hexfile, cleaned_data, auto_serial_number):
+        '''Write HEXFILE to the flash to a chip of type CHIP_TYPE.  The
+        HEXFILE contains the data passed in CLEANED_DATA.
+        AUTO_SERILA_NUMBER must be True if the serial numbers are
+        being assigned automatically, False otherwise.
+
+        Returns True on success, False on any failure.
+        '''
         self.cmd_start([PATH_AVRDUDE, '-ctc2030', '-p%s' % chip_type,
                         '-Uflash:w:%s' % hexfile], [], logfile=self.console)
         for _ in self.cmd:
             pass	# consume output until program is done...
         self.cmd_done()
-        if self.cmd.exit_status == 0:
-            self.log('Success: CTid board has been programmed with '
-                     'serial number %d.' % cleaned_data['sn'])
-            self.sn.commit(cleaned_data['sn'], cleaned_data)
-            self.update_sn()
-        else:
+        if self.cmd.exit_status != 0:
             self.show_cmd_error(self.cmd, 'Programming Failed',
                                 'Failed to write the microcontroller flash.')
-            return
+            return False
+
+        self.log('Success: CTid board has been programmed with '
+                 'serial number %d.' % cleaned_data['sn'])
+        self.sn.commit(cleaned_data['sn'], cleaned_data, auto_serial_number)
+        return True
 
     def read_template_from_flash(self, chip_type):
         temp = tempfile.TemporaryFile(suffix='.bin', prefix='CTid-')
@@ -710,45 +738,60 @@ class UI(QMainWindow, Ui_MainWindow):
         length = flash[CTid_table_addr]
         table = flash[CTid_table_addr + 1:CTid_table_addr + 1 + length]
         table = ctid.Table(ctid.unstuff(table))
-        return self.ctid_table_to_template(table)
+        template = self.ctid_table_to_template(table)
+
+        if template is not None:
+            m = re.match(r'(.*)-[^-]+$', template['model'])
+            if m is not None:
+                # strip suffix off model-name:
+                template['model'] = m.group(1)
+        return template
 
     def reprogram_with_cal_params(self, chip_type):
         template = self.read_template_from_flash(chip_type)
         if template is None:
             return
 
-        self.sn.set_product(template['mfg'], template['model'])
+        sn = template['sn']
+        mfg = template['mfg']
+        model = template['model']
 
-        cal_params = self.sn.get_cal_data(template['sn'])
+        saved_product = self.sn.product
+        self.sn.set_product(mfg, model)
+        cal_params = self.sn.get_cal_data(sn)
         if cal_params is None:
+            self.sn.restore_product(saved_product)
             QMessageBox.critical(self, 'No Calibration Data Found',
                                  'No calibration data was found for '
-                                 'product %s with serial number %s.' %
-                                 (self.sn.product, template['sn']),
-                                 QMessageBox.Ok)
+                                 'product %s with serial number %s.' \
+                                 % (self.sn.product, sn), QMessageBox.Ok)
             return
 
         msg = ''
         for key, value in cal_params.items():
             msg += '\t%16s: %s\n' % (key, value)
 
-        self.console.write('\n\nFound calibration data for %s %s #%d:\n%s\n' %
-                           (ctid.mfg_short_name(template['mfg']),
-                            template['model'], template['sn'], msg))
-
         # merge the calibrated parameters with existing template:
         for name, value in cal_params.items():
             template[name] = value
+        model = template['model']
+
+        self.console.write('\n\nFound calibration data for %s %s #%d:\n%s\n' %
+                           (ctid.mfg_short_name(mfg), model, sn, msg))
 
         hexfile = self.create_hexfile(template,
                                       self.params[template['sensor_type']])
         if hexfile is None:
+            self.sn.restore_product(saved_product)
             self.mark_idle()
             return
-        self.write_flash(chip_type, hexfile, template)
+        self.write_flash(chip_type, hexfile, template,
+                         auto_serial_number=False)
         os.remove(hexfile)
         self.mark_idle()
 
+        self.sn.restore_product(saved_product)
+
     def read_flash(self, chip_type):
         template = self.read_template_from_flash(chip_type)
         if template is None:
@@ -816,7 +859,10 @@ class UI(QMainWindow, Ui_MainWindow):
 
         chip_type = self.chip_type()
         if chip_type is not None:
-            self.write_flash(chip_type, hexfile, cleaned_data)
+            auto_serial_number = self.auto_serial_checkbox.isChecked()
+            if self.write_flash(chip_type, hexfile, cleaned_data,
+                                auto_serial_number):
+                self.update_sn()
 
         os.remove(hexfile)
         self.mark_idle()
@@ -873,8 +919,14 @@ parser.add_argument('-F', '--full-screen', action='store_true',
                     help='Start application in full-screen mode.')
 parser.add_argument('-d', '--debug', action='store_const', const=logging.DEBUG,
                     dest='log_level', help='Show debug output.')
+parser.add_argument('-v', '--version', action='store_true',
+                    help='Output version information and exit.')
 args = parser.parse_args()
 
+if args.version:
+    print(VERSION_INFO)
+    sys.exit(0)
+
 log_level = logging.ERROR if args.log_level is None else args.log_level
 logging.basicConfig()
 log = logging.getLogger()	# get the root logger
