@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020 eGauge Systems LLC
+# Copyright (c) 2020-2021 eGauge Systems LLC
 #       1644 Conestoga St, Suite 2
 #       Boulder, CO 80301
 #       voice: 720-545-9767
@@ -64,14 +64,13 @@ class Manager(sn.Manager):
                                'Serial Number Selection',
                                ('Would you like to switch to %d '
                                 'as the next automatic serial number '
-                                'for this product '
-                                'or would you like to revert back to '
+                                'for this product?  If you press "No", '
+                                'the program will revert back to '
                                 'the old sequence and use %d '
-                                'as the next automatic serial number?'
+                                'as the next automatic serial number.'
                                 % (sn, old_sn)),
-                               'Switch to %d' % sn,
-                               'Revert back to %d' % old_sn)
-            if choice == 0:
+                               QMessageBox.Yes, QMessageBox.No)
+            if choice == QMessageBox.Yes:
                 self._reset_serial_number(sn)
 
     def deactivate(self):
@@ -84,11 +83,11 @@ class Manager(sn.Manager):
             super().set(sn)
         return sn
 
-    def commit(self, sn, info):
-        # If the serial number isn't our next automatic serial number,
-        # the user must have entered a serial number manually.
-        if sn == super().get():
+    def commit(self, sn, info, auto_serial_number):
+        if auto_serial_number:
             self._increment_serial_number()
+        else:
+            super().set(None)
 
     def _set_stepping(self, increment, first):
         if increment < 1:
